@@ -1,4 +1,4 @@
-# Copyright 2016-2020 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2021 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -26,13 +26,14 @@ import reframe.core.fields as fields
 import reframe.core.logging as logging
 import reframe.core.runtime as rt
 import reframe.utility as util
+import reframe.utility.jsonext as jsonext
 import reframe.utility.osext as osext
 import reframe.utility.sanity as sn
 import reframe.utility.typecheck as typ
 import reframe.utility.udeps as udeps
-from reframe.core.backends import (getlauncher, getscheduler)
+from reframe.core.backends import getlauncher, getscheduler
 from reframe.core.buildsystems import BuildSystemField
-from reframe.core.containers import ContainerPlatform, ContainerPlatformField
+from reframe.core.containers import ContainerPlatformField
 from reframe.core.deferrable import _DeferredExpression
 from reframe.core.exceptions import (BuildError, DependencyError,
                                      PipelineError, SanityError,
@@ -125,7 +126,7 @@ def final(fn):
     return _wrapped
 
 
-class RegressionTest(metaclass=RegressionTestMeta):
+class RegressionTest(jsonext.JSONSerializable, metaclass=RegressionTestMeta):
     '''Base class for regression tests.
 
     All regression tests must eventually inherit from this class.
@@ -162,7 +163,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #: The name of the test.
     #:
     #: :type: string that can contain any character except ``/``
-    name = fields.TypedField('name', typ.Str[r'[^\/]+'])
+    name = fields.TypedField(typ.Str[r'[^\/]+'])
 
     #: List of programming environments supported by this test.
     #:
@@ -179,8 +180,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:     .. versionchanged:: 2.17
     #:        Support for wildcards is dropped.
     #:
-    valid_prog_environs = fields.TypedField('valid_prog_environs',
-                                            typ.List[str], type(None))
+    valid_prog_environs = fields.TypedField(typ.List[str], type(None))
 
     #: List of systems supported by this test.
     #: The general syntax for systems is ``<sysname>[:<partname>]``.
@@ -189,14 +189,13 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    valid_systems = fields.TypedField('valid_systems',
-                                      typ.List[str], type(None))
+    valid_systems = fields.TypedField(typ.List[str], type(None))
 
     #: A detailed description of the test.
     #:
     #: :type: :class:`str`
     #: :default: ``self.name``
-    descr = fields.TypedField('descr', str)
+    descr = fields.TypedField(str)
 
     #: The path to the source file or source directory of the test.
     #:
@@ -214,7 +213,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`str`
     #: :default: ``''``
-    sourcepath = fields.TypedField('sourcepath', str)
+    sourcepath = fields.TypedField(str)
 
     #: The directory containing the test's resources.
     #:
@@ -244,7 +243,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:     .. versionchanged:: 3.0
     #:        Default value is now conditionally set to either ``'src'`` or
     #:        :class:`None`.
-    sourcesdir = fields.TypedField('sourcesdir', str, type(None))
+    sourcesdir = fields.TypedField(str, type(None))
 
     #: .. versionadded:: 2.14
     #:
@@ -261,7 +260,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`str` or :class:`reframe.core.buildsystems.BuildSystem`.
     #: :default: :class:`None`.
-    build_system = BuildSystemField('build_system', type(None))
+    build_system = BuildSystemField(type(None))
 
     #: .. versionadded:: 3.0
     #:
@@ -273,15 +272,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    prebuild_cmds = fields.TypedField('prebuild_cmds', typ.List[str])
-
-    #: .. deprecated:: 3.0
-    #:
-    #: Use :attr:`prebuild_cmds` instead.
-    prebuild_cmd = fields.DeprecatedField(
-        fields.TypedField('prebuild_cmds', typ.List[str]),
-        "'prebuild_cmd' is deprecated; please use 'prebuild_cmds' instead"
-    )
+    prebuild_cmds = fields.TypedField(typ.List[str])
 
     #: .. versionadded:: 3.0
     #:
@@ -293,27 +284,19 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    postbuild_cmds = fields.TypedField('postbuild_cmds', typ.List[str])
-
-    #: .. deprecated:: 3.0
-    #:
-    #: Use :attr:`postbuild_cmds` instead.
-    postbuild_cmd = fields.DeprecatedField(
-        fields.TypedField('postbuild_cmds', typ.List[str]),
-        "'postbuild_cmd' is deprecated; please use 'postbuild_cmds' instead"
-    )
+    postbuild_cmds = fields.TypedField(typ.List[str])
 
     #: The name of the executable to be launched during the run phase.
     #:
     #: :type: :class:`str`
     #: :default: ``os.path.join('.', self.name)``
-    executable = fields.TypedField('executable', str)
+    executable = fields.TypedField(str)
 
     #: List of options to be passed to the :attr:`executable`.
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    executable_opts = fields.TypedField('executable_opts', typ.List[str])
+    executable_opts = fields.TypedField(typ.List[str])
 
     #: .. versionadded:: 2.20
     #:
@@ -337,8 +320,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #: :type: :class:`str` or
     #:     :class:`reframe.core.containers.ContainerPlatform`.
     #: :default: :class:`None`.
-    container_platform = ContainerPlatformField('container_platform',
-                                                type(None))
+    container_platform = ContainerPlatformField(type(None))
 
     #: .. versionadded:: 3.0
     #:
@@ -350,15 +332,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    prerun_cmds = fields.TypedField('prerun_cmds', typ.List[str])
-
-    #: .. deprecated:: 3.0
-    #:
-    #: Use :attr:`prerun_cmds` instead.
-    pre_run = fields.DeprecatedField(
-        fields.TypedField('prerun_cmds', typ.List[str]),
-        "'pre_run' is deprecated; please use 'prerun_cmds' instead"
-    )
+    prerun_cmds = fields.TypedField(typ.List[str])
 
     #: .. versionadded:: 3.0
     #:
@@ -369,15 +343,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    postrun_cmds = fields.TypedField('postrun_cmds', typ.List[str])
-
-    #: .. deprecated:: 3.0
-    #:
-    #: Use :attr:`postrun_cmds` instead.
-    post_run = fields.DeprecatedField(
-        fields.TypedField('postrun_cmds', typ.List[str]),
-        "'post_run' is deprecated; please use 'postrun_cmds' instead"
-    )
+    postrun_cmds = fields.TypedField(typ.List[str])
 
     #: List of files to be kept after the test finishes.
     #:
@@ -397,7 +363,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #: .. versionchanged:: 3.3
     #:    This field accepts now also file glob patterns.
     #:
-    keep_files = fields.TypedField('keep_files', typ.List[str])
+    keep_files = fields.TypedField(typ.List[str])
 
     #: List of files or directories (relative to the :attr:`sourcesdir`) that
     #: will be symlinked in the stage directory and not copied.
@@ -407,7 +373,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    readonly_files = fields.TypedField('readonly_files', typ.List[str])
+    readonly_files = fields.TypedField(typ.List[str])
 
     #: Set of tags associated with this test.
     #:
@@ -415,7 +381,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`Set[str]`
     #: :default: an empty set
-    tags = fields.TypedField('tags', typ.Set[str])
+    tags = fields.TypedField(typ.Set[str])
 
     #: List of people responsible for this test.
     #:
@@ -423,7 +389,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    maintainers = fields.TypedField('maintainers', typ.List[str])
+    maintainers = fields.TypedField(typ.List[str])
 
     #: Mark this test as a strict performance test.
     #:
@@ -433,7 +399,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: boolean
     #: :default: :class:`True`
-    strict_check = fields.TypedField('strict_check', bool)
+    strict_check = fields.TypedField(bool)
 
     #: Number of tasks required by this test.
     #:
@@ -459,7 +425,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: .. |--flex-alloc-nodes| replace:: :attr:`--flex-alloc-nodes`
     #: .. _--flex-alloc-nodes: manpage.html#cmdoption-flex-alloc-nodes
-    num_tasks = fields.TypedField('num_tasks', int)
+    num_tasks = fields.TypedField(int)
 
     #: Number of tasks per node required by this test.
     #:
@@ -467,8 +433,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: integral or :class:`None`
     #: :default: :class:`None`
-    num_tasks_per_node = fields.TypedField('num_tasks_per_node',
-                                           int, type(None))
+    num_tasks_per_node = fields.TypedField(int, type(None))
 
     #: Number of GPUs per node required by this test.
     #: This attribute is translated internally to the ``_rfm_gpu`` resource.
@@ -477,7 +442,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: integral
     #: :default: ``0``
-    num_gpus_per_node = fields.TypedField('num_gpus_per_node', int)
+    num_gpus_per_node = fields.TypedField(int)
 
     #: Number of CPUs per task required by this test.
     #:
@@ -485,7 +450,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: integral or :class:`None`
     #: :default: :class:`None`
-    num_cpus_per_task = fields.TypedField('num_cpus_per_task', int, type(None))
+    num_cpus_per_task = fields.TypedField(int, type(None))
 
     #: Number of tasks per core required by this test.
     #:
@@ -493,8 +458,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: integral or :class:`None`
     #: :default: :class:`None`
-    num_tasks_per_core = fields.TypedField('num_tasks_per_core',
-                                           int, type(None))
+    num_tasks_per_core = fields.TypedField(int, type(None))
 
     #: Number of tasks per socket required by this test.
     #:
@@ -502,8 +466,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: integral or :class:`None`
     #: :default: :class:`None`
-    num_tasks_per_socket = fields.TypedField('num_tasks_per_socket',
-                                             int, type(None))
+    num_tasks_per_socket = fields.TypedField(int, type(None))
 
     #: Specify whether this tests needs simultaneous multithreading enabled.
     #:
@@ -511,8 +474,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: boolean or :class:`None`
     #: :default: :class:`None`
-    use_multithreading = fields.TypedField('use_multithreading',
-                                           bool, type(None))
+    use_multithreading = fields.TypedField(bool, type(None))
 
     #: .. versionadded:: 3.0
     #:
@@ -522,19 +484,19 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`str` or :class:`datetime.timedelta`
     #: :default: :class:`None`
-    max_pending_time = fields.TimerField('max_pending_time', type(None))
+    max_pending_time = fields.TimerField(type(None))
 
     #: Specify whether this test needs exclusive access to nodes.
     #:
     #: :type: boolean
     #: :default: :class:`False`
-    exclusive_access = fields.TypedField('exclusive_access', bool)
+    exclusive_access = fields.TypedField(bool)
 
     #: Always execute this test locally.
     #:
     #: :type: boolean
     #: :default: :class:`False`
-    local = fields.TypedField('local', bool)
+    local = fields.TypedField(bool)
 
     #: The set of reference values for this test.
     #:
@@ -567,7 +529,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:        The measurement unit is required. The user should explicitly
     #:        specify :class:`None` if no unit is available.
     reference = fields.ScopedDictField(
-        'reference', typ.Tuple[object, object, object, object])
+        typ.Tuple[object, object, object, object])
     # FIXME: There is not way currently to express tuples of `float`s or
     # `None`s, so we just use the very generic `object`
 
@@ -593,8 +555,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:       ::
     #:
     #:           self.sanity_patterns = sn.assert_found(r'.*', self.stdout)
-    sanity_patterns = fields.TypedField('sanity_patterns',
-                                        _DeferredExpression, type(None))
+    sanity_patterns = fields.TypedField(_DeferredExpression, type(None))
 
     #: Patterns for verifying the performance of this test.
     #:
@@ -609,7 +570,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:     :class:`None` is also allowed.
     #: :default: :class:`None`
     perf_patterns = fields.TypedField(
-        'perf_patterns', typ.Dict[str, _DeferredExpression], type(None))
+        typ.Dict[str, _DeferredExpression], type(None))
 
     #: List of modules to be loaded before running this test.
     #:
@@ -617,7 +578,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    modules = fields.TypedField('modules', typ.List[str])
+    modules = fields.TypedField(typ.List[str])
 
     #: Environment variables to be set before running this test.
     #:
@@ -625,7 +586,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:
     #: :type: :class:`Dict[str, str]`
     #: :default: ``{}``
-    variables = fields.TypedField('variables', typ.Dict[str, str])
+    variables = fields.TypedField(typ.Dict[str, str])
 
     #: Time limit for this test.
     #:
@@ -649,7 +610,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:       - The old syntax using a ``(h, m, s)`` tuple is dropped.
     #:       - Support of `timedelta` objects is dropped.
     #:       - Number values are now accepted.
-    time_limit = fields.TimerField('time_limit', type(None))
+    time_limit = fields.TimerField(type(None))
 
     #: .. versionadded:: 2.8
     #:
@@ -717,8 +678,8 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #:    .. versionchanged:: 2.9
     #:       A new more powerful syntax was introduced
     #:       that allows also custom job script directive prefixes.
-    extra_resources = fields.TypedField('extra_resources',
-                                        typ.Dict[str, typ.Dict[str, object]])
+    extra_resources = fields.TypedField(
+        typ.Dict[str, typ.Dict[str, object]])
 
     #: .. versionadded:: 3.3
     #:
@@ -733,14 +694,20 @@ class RegressionTest(metaclass=RegressionTestMeta):
     #: appropriate sanity check.
     #:
     #: :type: boolean : :default: :class:`True`
-    build_locally = fields.TypedField('build_locally', bool)
+    build_locally = fields.TypedField(bool)
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, *args, _rfm_use_params=False, **kwargs):
         obj = super().__new__(cls)
 
+        # Set the test parameters in the object
+        cls._init_params(obj, _rfm_use_params)
+
         # Create a test name from the class name and the constructor's
         # arguments
         name = cls.__qualname__
+        name += obj._append_parameters_to_name()
+
+        # or alternatively, if the parameterized test was defined the old way.
         if args or kwargs:
             arg_names = map(lambda x: util.toalphanum(str(x)),
                             itertools.chain(args, kwargs.values()))
@@ -766,6 +733,43 @@ class RegressionTest(metaclass=RegressionTestMeta):
     def __init__(self):
         pass
 
+    @classmethod
+    def _init_params(cls, obj, use_params=False):
+        '''Attach the test parameters as class attributes.
+
+        Create and initialize the regression test parameters as object
+        attributes. The values assigned to these parameters exclusively depend
+        on the use_params argument. If this is set to True, the current object
+        uses the parameter space iterator (see
+        :class  `reframe.core.pipeline.RegressionTest` and consumes a set of
+        parameter values (i.e. a point in the parameter space). Contrarily, if
+        use_params is False, the regression test parameters are initialized as
+        None.
+
+        :param use_param: bool that dictates whether an instance of the
+        :class `reframe.core.pipeline.RegressionTest` is to use the
+        parameter values defined in the parameter space.
+
+        :meta private:
+        '''
+        # Set the values of the test parameters (if any)
+        if use_params and cls._rfm_param_space.params:
+            # Consume the parameter space iterator
+            param_values = next(cls._rfm_param_space.unique_iter)
+            for index, key in enumerate(cls._rfm_param_space.params):
+                setattr(obj, key, param_values[index])
+        else:
+            # Otherwise init the params as None
+            for key in cls._rfm_param_space.params:
+                setattr(obj, key, None)
+
+    def _append_parameters_to_name(self):
+        if self._rfm_param_space.params:
+            return '_' + '_'.join([str(self.__dict__[key])
+                                   for key in self._rfm_param_space.params])
+        else:
+            return ''
+
     @classmethod
     def __init_subclass__(cls, *, special=False, pin_prefix=False, **kwargs):
         super().__init_subclass__(**kwargs)
@@ -971,9 +975,10 @@ class RegressionTest(metaclass=RegressionTestMeta):
         This attribute is evaluated lazily, so it can by used inside sanity
         expressions.
 
-        :type: :class:`str`.
+        :type: :class:`str` or :class:`None` if a run job has not yet been
+            created.
         '''
-        return self._job.stdout
+        return self.job.stdout if self.job else None
 
     @property
     @sn.sanity_function
@@ -985,9 +990,10 @@ class RegressionTest(metaclass=RegressionTestMeta):
         This attribute is evaluated lazily, so it can by used inside sanity
         expressions.
 
-        :type: :class:`str`.
+        :type: :class:`str` or :class:`None` if a run job has not yet been
+            created.
         '''
-        return self._job.stderr
+        return self.job.stderr if self.job else None
 
     @property
     def build_job(self):
@@ -996,12 +1002,12 @@ class RegressionTest(metaclass=RegressionTestMeta):
     @property
     @sn.sanity_function
     def build_stdout(self):
-        return self._build_job.stdout
+        return self.build_job.stdout if self.build_job else None
 
     @property
     @sn.sanity_function
     def build_stderr(self):
-        return self._build_job.stderr
+        return self.build_job.stderr if self.build_job else None
 
     def info(self):
         '''Provide live information for this test.
@@ -1124,6 +1130,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         self._current_partition = partition
         self._current_environ = environ
@@ -1140,8 +1151,7 @@ class RegressionTest(metaclass=RegressionTestMeta):
         self.logger.debug(f'Symlinking files: {self.readonly_files}')
         try:
             osext.copytree_virtual(
-                path, self._stagedir, self.readonly_files, symlinks=True,
-                dirs_exist_ok=True
+                path, self._stagedir, self.readonly_files, dirs_exist_ok=True
             )
         except (OSError, ValueError, TypeError) as e:
             raise PipelineError('copying of files failed') from e
@@ -1165,6 +1175,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         if not self._current_environ:
             raise PipelineError('no programming environment set')
@@ -1262,6 +1277,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         self._build_job.wait()
 
@@ -1284,6 +1304,12 @@ class RegressionTest(metaclass=RegressionTestMeta):
               special test. See `here
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
+
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         if not self.current_system or not self._current_partition:
             raise PipelineError('no system or system partition is set')
@@ -1389,6 +1415,12 @@ class RegressionTest(metaclass=RegressionTestMeta):
            <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
            more details.
 
+
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         if not self._job:
             return True
@@ -1419,6 +1451,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
            <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
            more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         self._job.wait()
 
@@ -1460,6 +1497,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         if rt.runtime().get_option('general/0/trap_job_errors'):
             sanity_patterns = [
@@ -1493,6 +1535,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         if self.perf_patterns is None:
             return
@@ -1611,6 +1658,11 @@ class RegressionTest(metaclass=RegressionTestMeta):
               <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
               more details.
 
+           .. versionchanged:: 3.4
+              Overriding this method directly in no longer allowed. See `here
+              <migration_2_to_3.html#force-override-a-pipeline-method>`__ for
+              more details.
+
         '''
         aliased = os.path.samefile(self._stagedir, self._outputdir)
         if aliased:
@@ -1792,6 +1844,10 @@ class RegressionTest(metaclass=RegressionTestMeta):
     def __hash__(self):
         return hash(self.name)
 
+    def __rfm_json_decode__(self, json):
+        # 'tags' are decoded as list, so we convert them to a set
+        self.tags = set(json['tags'])
+
 
 class RunOnlyRegressionTest(RegressionTest, special=True):
     '''Base class for run-only regression tests.
@@ -1874,12 +1930,12 @@ class CompileOnlyRegressionTest(RegressionTest, special=True):
     @property
     @sn.sanity_function
     def stdout(self):
-        return self._build_job.stdout
+        return self.build_job.stdout if self.build_job else None
 
     @property
     @sn.sanity_function
     def stderr(self):
-        return self._build_job.stderr
+        return self.build_job.stderr if self.build_job else None
 
     def run(self):
         '''The run stage of the regression test pipeline.
