@@ -1,4 +1,4 @@
-# Copyright 2016-2020 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2021 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -12,10 +12,10 @@ import functools
 from datetime import datetime
 
 import reframe.core.config as config
-import reframe.core.fields as fields
-import reframe.utility.os_ext as os_ext
+import reframe.utility.osext as osext
 from reframe.core.environments import (Environment, snapshot)
 from reframe.core.exceptions import ReframeFatalError
+from reframe.core.logging import getlogger
 from reframe.core.systems import System
 
 
@@ -36,7 +36,7 @@ class RuntimeContext:
     def _makedir(self, *dirs, wipeout=False):
         ret = os.path.join(*dirs)
         if wipeout:
-            os_ext.rmtree(ret, ignore_errors=True)
+            osext.rmtree(ret, ignore_errors=True)
 
         os.makedirs(ret, exist_ok=True)
         return ret
@@ -80,19 +80,19 @@ class RuntimeContext:
 
     @property
     def prefix(self):
-        return os_ext.expandvars(
+        return osext.expandvars(
             self.site_config.get('systems/0/prefix')
         )
 
     @property
     def stagedir(self):
-        return os_ext.expandvars(
+        return osext.expandvars(
             self.site_config.get('systems/0/stagedir')
         )
 
     @property
     def outputdir(self):
-        return os_ext.expandvars(
+        return osext.expandvars(
             self.site_config.get('systems/0/outputdir')
         )
 
@@ -104,7 +104,7 @@ class RuntimeContext:
             if h['type'] == 'filelog':
                 break
 
-        return os_ext.expandvars(
+        return osext.expandvars(
             self.site_config.get(f'logging/0/handlers_perflog/{i}/basedir')
         )
 
@@ -141,12 +141,18 @@ class RuntimeContext:
 
     def make_stagedir(self, *dirs):
         wipeout = self.get_option('general/0/clean_stagedir')
-        return self._makedir(self.stage_prefix,
-                             *self._format_dirs(*dirs), wipeout=wipeout)
+        ret = self._makedir(self.stage_prefix,
+                            *self._format_dirs(*dirs), wipeout=wipeout)
+        getlogger().debug(
+            f'Created stage directory {ret!r} [clean_stagedir: {wipeout}]'
+        )
+        return ret
 
     def make_outputdir(self, *dirs):
-        return self._makedir(self.output_prefix,
-                             *self._format_dirs(*dirs), wipeout=True)
+        ret = self._makedir(self.output_prefix,
+                            *self._format_dirs(*dirs), wipeout=True)
+        getlogger().debug(f'Created output directory {ret!r}')
+        return ret
 
     @property
     def modules_system(self):
@@ -205,16 +211,19 @@ def loadenv(*environs):
     env_snapshot = snapshot()
     commands = []
     for env in environs:
-        for m in env.modules:
-            conflicted = modules_system.load_module(m, force=True)
-            for c in conflicted:
-                commands += modules_system.emit_unload_commands(c)
+        for mod in env.modules_detailed:
+            load_seq = modules_system.load_module(**mod, force=True)
+            for m, conflicted in load_seq:
+                for c in conflicted:
+                    commands += modules_system.emit_unload_commands(c)
 
-            commands += modules_system.emit_load_commands(m)
+                commands += modules_system.emit_load_commands(
+                    m, mod['collection']
+                )
 
         for k, v in env.variables.items():
-            os.environ[k] = os_ext.expandvars(v)
-            commands.append('export %s=%s' % (k, v))
+            os.environ[k] = osext.expandvars(v)
+            commands.append(f'export {k}={v}')
 
     return env_snapshot, commands
 
@@ -240,7 +249,7 @@ def is_env_loaded(environ):
     '''
     is_module_loaded = runtime().modules_system.is_module_loaded
     return (all(map(is_module_loaded, environ.modules)) and
-            all(os.environ.get(k, None) == os_ext.expandvars(v)
+            all(os.environ.get(k, None) == osext.expandvars(v)
                 for k, v in environ.variables.items()))
 
 
@@ -277,7 +286,7 @@ class temp_runtime:
             _runtime_context = None
         else:
             site_config = config.load_config(config_file)
-            site_config.select_subconfig(sysname)
+            site_config.select_subconfig(sysname, ignore_resolve_errors=True)
             for opt, value in options.items():
                 site_config.add_sticky_option(opt, value)
 
