@@ -1,4 +1,4 @@
-# Copyright 2016-2020 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2021 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -13,10 +13,11 @@ import time
 import reframe.core.fields as fields
 import reframe.core.runtime as runtime
 import reframe.core.shell as shell
+import reframe.utility.jsonext as jsonext
 import reframe.utility.typecheck as typ
 from reframe.core.exceptions import JobError, JobNotStartedError
 from reframe.core.launchers import JobLauncher
-from reframe.core.logging import getlogger
+from reframe.core.logging import getlogger, DEBUG2
 
 
 class JobScheduler(abc.ABC):
@@ -102,8 +103,16 @@ class JobScheduler(abc.ABC):
         :meta private:
         '''
 
+    def log(self, message, level=DEBUG2):
+        '''Convenience method for logging debug messages from the scheduler
+        backends.
 
-class Job:
+        :meta private:
+        '''
+        getlogger().log(level, f'[S] {self.registered_name}: {message}')
+
+
+class Job(jsonext.JSONSerializable):
     '''A job descriptor.
 
     A job descriptor is created by the framework after the "setup" phase and
@@ -114,24 +123,19 @@ class Job:
 
     '''
 
-    num_tasks = fields.TypedField('num_tasks', int)
-    num_tasks_per_node = fields.TypedField('num_tasks_per_node',
-                                           int,  type(None))
-    num_tasks_per_core = fields.TypedField('num_tasks_per_core',
-                                           int,  type(None))
-    num_tasks_per_socket = fields.TypedField('num_tasks_per_socket',
-                                             int,  type(None))
-
-    num_cpus_per_task = fields.TypedField('num_cpus_per_task',
-                                          int,  type(None))
-    use_smt = fields.TypedField('use_smt', bool,  type(None))
-    time_limit = fields.TimerField('time_limit', type(None))
+    num_tasks = fields.TypedField(int)
+    num_tasks_per_node = fields.TypedField(int, type(None))
+    num_tasks_per_core = fields.TypedField(int, type(None))
+    num_tasks_per_socket = fields.TypedField(int, type(None))
+    num_cpus_per_task = fields.TypedField(int, type(None))
+    use_smt = fields.TypedField(bool, type(None))
+    time_limit = fields.TimerField(type(None))
 
     #: Options to be passed to the backend job scheduler.
     #:
     #: :type: :class:`List[str]`
     #: :default: ``[]``
-    options = fields.TypedField('options', typ.List[str])
+    options = fields.TypedField(typ.List[str])
 
     #: The (parallel) program launcher that will be used to launch the
     #: (parallel) executable of this job.
@@ -153,7 +157,7 @@ class Job:
     #:        self.job.launcher = getlauncher('local')()
     #:
     #: :type: :class:`reframe.core.launchers.JobLauncher`
-    launcher = fields.TypedField('launcher', JobLauncher)
+    launcher = fields.TypedField(JobLauncher)
 
     # The sched_* arguments are exposed also to the frontend
     def __init__(self,
@@ -176,7 +180,8 @@ class Job:
         self.num_cpus_per_task = None
         self.use_smt = None
         self.time_limit = None
-        self.options = list(sched_options) if sched_options else []
+        self.cli_options = list(sched_options) if sched_options else []
+        self.options = []
 
         self._name = name
         self._workdir = workdir
@@ -317,7 +322,7 @@ class Job:
 
         .. versionadded:: 2.17
 
-        :type:`List[str]` or :type:`None`
+        :type: :class:`List[str]` or :class:`None`
         '''
         return self._nodelist
 
@@ -328,6 +333,7 @@ class Job:
     def prepare(self, commands, environs=None, **gen_opts):
         environs = environs or []
         if self.num_tasks <= 0:
+            getlogger().debug(f'[F] Flexible node allocation requested')
             num_tasks_per_node = self.num_tasks_per_node or 1
             min_num_tasks = (-self.num_tasks if self.num_tasks else
                              num_tasks_per_node)
@@ -336,17 +342,17 @@ class Job:
                 guessed_num_tasks = self.guess_num_tasks()
             except NotImplementedError as e:
                 raise JobError('flexible node allocation is not supported by '
-                               'this backend') from e
+                               'this scheduler backend') from e
 
             if guessed_num_tasks < min_num_tasks:
                 raise JobError(
                     'could not satisfy the minimum task requirement: '
                     'required %s, found %s' %
-                    (min_num_tasks, guessed_num_tasks))
+                    (min_num_tasks, guessed_num_tasks)
+                )
 
             self.num_tasks = guessed_num_tasks
-            getlogger().debug('flex_alloc_nodes: setting num_tasks to %s' %
-                              self.num_tasks)
+            getlogger().debug(f'[F] Setting num_tasks to {self.num_tasks}')
 
         with shell.generate_script(self.script_filename,
                                    **gen_opts) as builder:
@@ -365,8 +371,9 @@ class Job:
             return self.sched_flex_alloc_nodes * num_tasks_per_node
 
         available_nodes = self.scheduler.allnodes()
-        getlogger().debug('flex_alloc_nodes: total available nodes: %s ' %
-                          len(available_nodes))
+        getlogger().debug(
+            f'[F] Total available nodes: {len(available_nodes)}'
+        )
 
         # Try to guess the number of tasks now
         available_nodes = self.scheduler.filternodes(self, available_nodes)
@@ -374,7 +381,7 @@ class Job:
             available_nodes = {n for n in available_nodes
                                if n.in_state(self.sched_flex_alloc_nodes)}
             getlogger().debug(
-                f'flex_alloc_nodes: selecting nodes in state '
+                f'[F] Selecting nodes in state '
                 f'{self.sched_flex_alloc_nodes!r}: '
                 f'available nodes now: {len(available_nodes)}'
             )
